@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 
 # Copyright (C) 2014-2020, IONOS SE
-# Author: Benjamin Drung <benjamin.drung@cloud.ionos.com>
+# Author: Benjamin Drung <benjamin.drung@ionos.com>
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -130,13 +130,12 @@ def cache_file(cache_dir, source):
             logger.info("Copy cached download to %s...", destination)
         else:
             logger.info("Save download to %s...", destination)
-        cached_file = open(destination, "wb")
-        cached_file.write(content)
-        cached_file.close()
+        with open(destination, "wb") as cached_file:
+            cached_file.write(content)
     elif source.startswith("rsync:"):
         call_command(["rsync", "--no-motd", source, destination])
     else:
-        raise Exception("No download handler for file '{source}' found.".format(source=source))
+        raise Exception(f"No download handler for file '{source}' found.")
     return relative_destination
 
 
@@ -159,13 +158,12 @@ def download_and_publish(config, image, source, filename):
             logger.info("Copy cached download to %s...", destination)
         else:
             logger.info("Save download to %s...", destination)
-        cached_file = open(destination, "wb")
-        cached_file.write(content)
-        cached_file.close()
+        with open(destination, "wb") as cached_file:
+            cached_file.write(content)
     elif source.startswith("rsync:"):
         call_command(["rsync", "--no-motd", source, destination])
     else:
-        raise Exception("No download handler for file '{source}' found.".format(source=source))
+        raise Exception(f"No download handler for file '{source}' found.")
     return destination
 
 
@@ -177,9 +175,7 @@ def check_one_partition(partitions, image):
     """
     if len(partitions) != 1:
         if partitions:
-            msg = "{n} partitions ({partitions})".format(
-                n=len(partitions), partitions=", ".join(partitions)
-            )
+            msg = f"{len(partitions)} partitions ({', '.join(partitions)})"
         else:
             msg = "no partitions"
         logger = logging.getLogger(__logger_name__)
@@ -196,8 +192,8 @@ def parse_bytes(data):
     match = re.match(r"^([0-9]+)\s*([kMGTPE]?B|[KMGTPE]iB|[KMGTPE])$", data.strip())
     if not match:
         raise ValueError(
-            "Failed to parse bytes from '%s'. Please use SI or binary prefixes "
-            "for bytes (e.g. '2 GB' or '512 MiB')." % (data)
+            f"Failed to parse bytes from '{data}'. "
+            "Please use SI or binary prefixes for bytes (e.g. '2 GB' or '512 MiB')."
         )
 
     value = int(match.group(1))
@@ -231,19 +227,19 @@ def run_installation(config, image, image_name):  # pylint: disable=too-many-loc
     else:
         initrd = cache_file(cache_dir, config.get(image, "initrd"))
         kernel = cache_file(cache_dir, config.get(image, "linux"))
-        location = "{},kernel={},initrd={}".format(cache_dir, kernel, initrd)
+        location = f"{cache_dir},kernel={kernel},initrd={initrd}"
 
     append = config.get(image, "append", fallback="")
     if config.has_option(image, "preseed"):
         initrd_inject = config.get(image, "preseed")
-        append = "preseed/url=file:///{} {}".format(os.path.basename(initrd_inject), append)
+        append = f"preseed/url=file:///{os.path.basename(initrd_inject)} {append}"
     if config.has_option(image, "kickstart"):
         initrd_inject = config.get(image, "kickstart")
-        append = "ks=file:///{} {}".format(os.path.basename(initrd_inject), append)
-        append = "inst.ks=file:///{} {}".format(os.path.basename(initrd_inject), append)
+        append = f"ks=file:///{os.path.basename(initrd_inject)} {append}"
+        append = f"inst.ks=file:///{os.path.basename(initrd_inject)} {append}"
     if config.has_option(image, "yast"):
         initrd_inject = config.get(image, "yast")
-        append = "autoyast=file:///{} {}".format(os.path.basename(initrd_inject), append)
+        append = f"autoyast=file:///{os.path.basename(initrd_inject)} {append}"
 
     network = "user,model=virtio"
     if config.has_option(image, "mac"):
@@ -252,9 +248,9 @@ def run_installation(config, image, image_name):  # pylint: disable=too-many-loc
     graphics = "none"
     if config.has_option(image, "vnc"):
         listen, port = config.get(image, "vnc").split(":")
-        graphics = "vnc,port={}".format(5900 + int(port))
+        graphics = f"vnc,port={5900 + int(port)}"
         if listen:
-            graphics += ",listen={}".format(listen)
+            graphics += f",listen={listen}"
 
     session = get_session()
     cmd = [
@@ -296,13 +292,14 @@ def run_installation(config, image, image_name):  # pylint: disable=too-many-loc
         call_command(["virsh", "-c", session, "undefine", image_name])
 
 
-def open_as_user(filename):
+def open_as_user(filename, encoding):
+    # pylint: disable=consider-using-with
     try:
-        fileobject = open(filename)
+        fileobject = open(filename, encoding=encoding)
     except IOError as error:
         if error.errno == errno.EACCES:
             call_command(["chmod", "o+r", filename], as_root=True)
-            fileobject = open(filename)
+            fileobject = open(filename, encoding=encoding)
         else:
             raise
     return fileobject
@@ -362,7 +359,7 @@ def remove_logs(tmpdir, print_installer_logs):
         if print_installer_logs:
             logger = logging.getLogger(__logger_name__)
             for installer_log in installer_logs:
-                content = open_as_user(installer_log).read()
+                content = open_as_user(installer_log, encoding="utf-8").read()
                 logger.info("Content of /%s:\n%s", os.path.relpath(installer_log, tmpdir), content)
 
         # Remove installer logs
@@ -398,8 +395,8 @@ def post_installation(script_name, image, print_installer_logs):
     try:
         cmd = [LOSETUP, "-o", str(offset), "--show", "-f", image]
         logger.info("Calling %s", " ".join(cmd))
-        process = subprocess.Popen(cmd, stdout=subprocess.PIPE)
-        loopdev = process.communicate()[0].decode().strip()
+        with subprocess.Popen(cmd, stdout=subprocess.PIPE) as process:
+            loopdev = process.communicate()[0].decode().strip()
 
         tmpdir = tempfile.mkdtemp(prefix=script_name + ".")
         call_command(["mount", loopdev, tmpdir], as_root=True)
@@ -424,11 +421,11 @@ def post_installation(script_name, image, print_installer_logs):
 def create_hashsum(image):
     logger = logging.getLogger(__logger_name__)
     logger.info("Calculating SHA 256 sum of %s...", image)
-    sha256_sum = hashlib.sha256(open(image, "rb").read()).hexdigest()
+    with open(image, "rb") as image_file:
+        sha256_sum = hashlib.sha256(image_file.read()).hexdigest()
     logger.info("SHA 256 sum of %s: %s", image, sha256_sum)
-    checksum_file = open(image + ".sha256sum", "w")
-    checksum_file.write(sha256_sum + "  " + image + "\n")
-    checksum_file.close()
+    with open(image + ".sha256sum", "w", encoding="utf-8") as checksum_file:
+        checksum_file.write(sha256_sum + "  " + image + "\n")
 
 
 def create_qcow2(image, keep_raw_image):
@@ -514,7 +511,7 @@ def parse_args(argv):
         help="Image to build (date and suffix will be added).",
     )
     parser.add_argument(
-        "-c", "--cache-dir", help="Cache directory (default: %s)" % (get_default_cache_dir())
+        "-c", "--cache-dir", help=f"Cache directory (default: {get_default_cache_dir()})"
     )
     parser.add_argument(
         "-f", "--format", choices=["qcow2", "raw"], help="Image format to use (default: raw)"
